##
##  This file is part of The NWA Toolset.
##
##  Copyright (C) 2001-2002 Royal Library in Stockholm,
##                          Royal Library in Copenhagen,
##                          Helsinki University Library of Finland, 
##                          National Library of Norway,
##                          National and University Library of Iceland.
##
##  The NWA Toolset is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##
##  The NWA Toolset is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with The NWA Toolset; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##

# $Id: HTMLDocParser.pm,v 1.2 2004/10/04 08:51:13 johnerik Exp $

package NWA::HTML::HTMLDocParser;

use strict;
use vars qw($config %linkElements @ElmEndStack $VERSION @ISA @EXPORT @EXPORT_OK);

use Carp qw/confess/;
use CGI qw(:standard);
use Exporter ();
use URI::URL();
use HTML::Parser();
#use lib qw(/home/svc/development/NWA-cvs/nwa/test_impl/perl/);

#use HTML::Tagset (); # to be able to use the hash %linkelements

@ISA = qw( HTML::Parser Exporter ); 
@EXPORT = qw();
@EXPORT_OK = qw();
$VERSION = '1.00';

@ElmEndStack = (); 

#HTML::Tagset 3.03 (Contains hash-table %linkElements), 
#The following is the hash table as of (v. 3.03 of HTML::Tagset)
## modified by adding 

# 'param'   => ['value'], # notice, that only contains an URL, 
#                         # if name-attribute = 'url
#                         # so tag param is a special case
#);

%linkElements =
(
 'a'       => ['href'],
 'applet'  => ['archive', 'codebase', 'code'],
 'area'    => ['href'],
 'base'    => ['href'],
 'bgsound' => ['src'],
 'blockquote' => ['cite'],
 'body'    => ['background'],
 'del'     => ['cite'],
 'embed'   => ['pluginspage', 'src'],
 'form'    => ['action'],
 'frame'   => ['src', 'longdesc'],
 'iframe'  => ['src', 'longdesc'],
 'ilayer'  => ['background'],
 'img'     => ['src', 'dynsrc', 'lowsrc', 'longdesc', 'usemap'],
 'input'   => ['src', 'usemap'],
 'ins'     => ['cite'],
 'isindex' => ['action'],
 'head'    => ['profile'],
 'layer'   => ['background', 'src'],
 'link'    => ['href'],
 #'object'  => ['classid', 'codebase', 'data', 'archive', 'usemap'],
 'object'  => ['classid', 'codebase', 'data', 'archive', 'usemap'],
 'q'       => ['cite'],
 #'script'  => ['src', 'for'],
'script'  => ['src', 'for','archive'], ## is this correct 
'table'   => ['background'],
 'td'      => ['background'],
 'th'      => ['background'],
 'tr'      => ['background'],
 #'xmp'     => ['href'],
 'param'   => ['value'], # notice, that only contains an URL, 
                         # if name-attribute = 'url
                         # so tag param is a special case
);

#==================== Constructor  ======================================

## which params er necessary to initiate new NWA::HTML::HTMLDocParser object

## - $timenum 
## - $url
## - $callback

sub new {
  my $class = shift; 
  my $url =shift;
  my $timenum = shift;
  my $callback = shift;

  # check if all necessary arguments are there
  # Abort, if that is not the case!

  if (!defined($url)) 
	{ confess ('HTMLDocParser::new - missing parameter url') }
  elsif (!defined($timenum)) 
	{ confess ('HTMLDocParser::new - missing parameter timenum') }
  elsif (!defined($callback)) 
        { confess ('HTMLDocParser::new - missing parameter callback')
    }
 
 ## Initialize object

 my $self = new HTML::Parser;
  
  $self->{url} = $url;
  $self->{timenum} = $timenum;
  $self->{callback} = $callback;
  
  # base-url can also be set, when the parser finds a baseref statement

  $self->{baseurl} = $self->{url};

  ## Generering af baseurl forudstter at alle url'er overholder flgende:
  ## <protocol>://<host>/<path>/<file> hvis url'en peger p en fil
  ## <protocol>://<host>/<path>/ hvis url'en peger p en folder

  $self->{base} = $self->{baseurl};
  $self->{base} =~ s/(.*\/).*/$1/;

  bless ($self,$class);
  
  return $self;
}

#######################################################
# function: getUrl
# input: an URL path ($path)
# output: an hash containing information about the URL
# 
# Most of the output is made by use of the CPAN module
# URI::URL

sub getURL {
  my ($self, $path) = @_;

  my $fragment = '';

  ## Split URL in document and document-fragment: e.g.  
  ## index.html#start -> 'index.html' and  'start'

  ($path, $fragment) = split /#/, $path;

  if (!defined($fragment)) { $fragment = ''}

  my $urlobj = URI::URL->new($path, $self->{base});

  my %retval = ( 'absurl' => ($urlobj->abs)->as_string, 
                  'path' => $path,
                  'fragment' => $fragment, 
                  'host' => URI::URL->new($self->{base})->host,
                  'urlobj' => $urlobj
                 );
  return %retval;
}

## extractURLs

## This function must be modified to extract more than URL per tag
## arguments: $self (reference to object)
##            $attr (pointer to hash with attributes given to current tag), 
##            $tag (current HTML tag in lowercase)
### Here we use the information in hash-table HTML::TagSet::linkelements

##          
## returns: list of hash of url_paths: (attribute, url) pairs

sub extractURLs {

  my ($self, $attr, $tag) = @_;
   
  my %url_paths = ();

  my $linkelement = undef;
  my $found_link = 0;
  my @possible_tag_linkelements = (); 
  
  ## Only do anything, if any linkelements exists for $tag in 
  # hash %linkElements
  
  if ($linkElements{$tag}){
      #print "\nDEBUG: $tag can have links\n";
      ## $tag can have linkelements
      #@possible_tag_linkelements = $linkElements{$tag};

      @possible_tag_linkelements = $linkElements{$tag};

      my $number_of_possible_linkelements = 
	  scalar @possible_tag_linkelements;      

      for (my $i = 0; $i < $number_of_possible_linkelements; $i++) {
	  my $cur_element = $linkElements{$tag}[$i];

	  if (exists($attr->{$cur_element})) {
	      $url_paths{$cur_element} = $attr->{$cur_element};
	  }

      }
  }

  # make special case for 'param' tag
  if ($tag eq 'param') {
      if (exists($attr->{'name'})) {

	  if (lc ($attr->{'name'}) eq 'url') { # assure  $attr->{'name'} is lowercase  
	      # remove entry 'name' in %url_paths
	      # insert entry 'value' in %url_paths
	      $url_paths{'value'} = $attr->{'value'};
	      delete $url_paths{'name'};
	  }
	  else {
	      if (exists ($url_paths{'name'})) {
		  delete $url_paths{'name'};
	      }
	  }
      }
  }	
  #print "\nDEBUG: found link\n" if ($found_link);

  return %url_paths;
      
     
}

### Auxiliary functions

sub ElmEndStackPush($$){
  my ($self, $prefix, $suffix) = @_;

  $self->{callback}->($prefix);
  push @ElmEndStack, $suffix;
}

sub ElmEndStackPop {
  my $self = shift;

  while (my $suffix = pop @ElmEndStack) {
    $self->{callback}->($suffix);
  }
}

sub MakeTag($$$) {
  my ($self, $tag, $attr, $attrseq) = @_;

  my $s = "<$tag";
  foreach my $a (@$attrseq) {
    $s .= " $a\=\"" . $attr->{$a} . '"';
  }
  $s .= ">";  
  return $s;
}

############# Routines that handle the tags ######################

# see include-file 'handletag.inc'  

sub split_url($) {
# Forudstter, at stien er: <codebasepath> / <code-file>
  my ($urlpath) = @_;

  my ($urldir, $urlfile) = $urlpath=~ /(.*\/)(.*)$/; 
  return ($urldir, $urlfile);
}

sub extractcodebase($$)  {
  my ($codeurl, $code) = @_;

  my ($ret, $codepart) =  $codeurl =~ /(.*)($code)$/;
  return $ret;
}

#### Funktioner kaldt af parseren
sub declaration {
  my ($self, $decl) = @_;
  
  my $s = "<!$decl>";
  $self->{callback}->($s); 
}

sub process_tag {
  my ($self, $tag, $attr, $attrseq, $origtext, %paths) = @_; 

  my %tag_functions = 
(  
 'a'       => \&handleImgTableTrTdThA,  ## before: \&handleA
 'applet'  => \&handleApplet,
 'area'    => \&handleAreaLinkScriptFrameIframeLayerIlayer, ## before \&handleArea 
 'base'    => \&handleBase,
 'bgsound' => \&handleBgsound,
 'blockquote' => \&handleBlockquote,
 'body'    => \&handleBody,
 'del'     => \&handleDel,
 'embed'   => \&handleEmbed,
 'form'    => \&handleForm,
 'frame'   => \&handleAreaLinkScriptFrameIframeLayerIlayer, ## before \&handleFrame 
 'iframe'  => \&handleAreaLinkScriptFrameIframeLayerIlayer, ## before \&handleIframe 
 'ilayer'  => \&handleAreaLinkScriptFrameIframeLayerIlayer, ## before \&handleIlayer 
 'img'     => \&handleImgTableTrTdThA, ## before \&handleImg,
# 'input'   => \&handleInput,
 'input'   => \&handleBody,
 'ins'     => \&handleIns,
 'isindex' => \&handleIsindex,
 'head'    => \&handleHead,
 'layer'   => \&handleAreaLinkScriptFrameIframeLayerIlayer,## before \&handleLayer
 'link'    => \&handleAreaLinkScriptFrameIframeLayerIlayer, ## before \&handleLink 
 'object'  => \&handleObject,
 'q'       => \&handleQ,
 'script'  => \&handleAreaLinkScriptFrameIframeLayerIlayer, ## before \&handleScript 
 'table'   => \&handleImgTableTrTdThA, ## before \&handleTable 
 'td'      => \&handleImgTableTrTdThA, ## before \&handleTd
 'th'      => \&handleImgTableTrTdThA, ## before \&handleTh
 'tr'      => \&handleImgTableTrTdThA, ## before \&handleTr
 'xmp'     => \&handleXmp,
 'param'   => \&handleParam,
);

my $tagHandler = $tag_functions{$tag}; 
if (!defined $tagHandler) {
    $self->{callback}->($origtext); ## maybe a default handler should be made
}
else {
    &$tagHandler($self, $tag, $attr, $attrseq, $origtext, %paths);
}

} ### end of sub process_tag #############



sub start {
  ## Start is called by the parser whenever it sees a start-tag 

  ## $tag  = name of tag
  ## $attr = pointer to attribute-hash
  ## $attrseq = reference to list of attributes? 
  ## $origtext = the whole contents of tag

  my ($self, $tag, $attr, $attrseq, $origtext) = @_; 

  ## Extract URLs from tag attributes 
  ## this should be returned in a pointer to hash
  ## Should these paths be absolute?
 
  
  my %paths = $self->extractURLs($attr, $tag);
  #my $paths_reference = $self->extractURLs($attr, $tag);
  #my %paths = %{$paths_reference}

  #print "DEBUG: Examining $tag\n";
  #print "DEBUG: $tag contains following linkattributes: \n";
  
  #no strict 'refs';
  my $key;
  #foreach my $key (keys %paths) {

  #foreach $key (keys %{$attr}) {
  foreach $key (keys %paths) {

     #print "DEBUG: attribute $key = " . $paths{$key} . " \n";    
}
  
   if (%paths) {
      #print "DEBUG: We now enter the if (defined %paths){\n"; 
    # Seen an element with links embedded 
    if ($tag eq 'applet') {  
       $self->{callback}->($origtext); # return the contents unmodified
   } 
    else {
	&process_tag($self, $tag, $attr, $attrseq, $origtext, %paths);
    }} 
  else {
      $self->{callback}->($origtext); # return the contents unmodified
  }
  
} ### end of sub start   ####



sub end {
  my ($self, $tag, $origtext) = @_;

  $self->ElmEndStackPop;
  $self->{callback}->($origtext);
}

sub text {
  my ($self, $text) = @_;
  
  $self->{callback}->($text); 
}

sub comment {
  my ($self, $comment) = @_;

  $self->{callback}->("<!--$comment-->"); 
}

sub handleBody($$$$$) { 
# Handling of Background-images: <body background=..>
    
  my ($self, $tag, $attr, $attrseq, $origtext, %paths) = @_;
  my $s = '';
  my $path;
  my $mode = 'inline';
  
  ### modify each of the linkelements contained by the tag
  ## print "\nDEBUG:: Calling handleBody";

  foreach my $attribute (keys %paths) {
      $path = $paths{$attribute};
      my %urlinfo = $self->getURL($path);
      my $url = $urlinfo{absurl};
      $attr->{$attribute} = "##P#R#E#F#I#X##TIME#$self->{timenum}##MODE#$mode##URL#$url##";
  }      
  $s .= $self->MakeTag($tag, $attr, $attrseq);
  $self->{callback}->($s);

} # end of sub handleBody()

sub handleBase($$$$$) { 
# Handling of BASE HREF tags : <base href='someurl'>
# OUTPUT: The tag itself is un-modified, but changes $self->{baseurl}
  
  ## print "\nDEBUG:: Calling handleBase";

  my ($self, $tag, $attr, $attrseq, $origtext, %paths) = @_;


  $self->{baseurl} = $attr->{'href'};
 
  my $s = $self->MakeTag($tag, $attr, $attrseq);
  $self->{callback}->($s);    

} # end of sub handleBase()

sub handleImgTableTrTdThA ($$$$$) { 
# Handling of tags IMG, TABLE, TR, TD TH and A
    
  my ($self, $tag, $attr, $attrseq, $origtext, %paths) = @_;
  my $s = '';
  my $path;
  my $mode = 'inline';
  if ($tag eq 'a') {
      $mode='standalone';
  }

  ### modify each of the linkelements contained by the tag
  ## print "\nDEBUG:: Calling handleBody";

  foreach my $attribute (keys %paths) {
      $path = $paths{$attribute};
      my %urlinfo = $self->getURL($path);
      my $url = $urlinfo{absurl};
      $attr->{$attribute} = "##P#R#E#F#I#X##TIME#$self->{timenum}##MODE#$mode##URL#$url##";
  }      
  $s .= $self->MakeTag($tag, $attr, $attrseq);
  $self->{callback}->($s);

} # end of sub handleImgTableTrTdThA ()



sub handleAreaLinkScriptFrameIframeLayerIlayer ($$$$$) { 

# Handling of tags AREA, LINK, SCRIPT, FRAME, IFRAME, LAYER, ILAYER
  my ($self, $tag, $attr, $attrseq, $origtext, %paths) = @_;
  my $s = '';
  my $path;
  my $mode = 'inline';
  if ($tag eq 'a') {
      $mode='standalone';
  }

  ### modify each of the linkelements contained by the tag
  ## print "\nDEBUG:: Calling handleBody";

  foreach my $attribute (keys %paths) {
      $path = $paths{$attribute};
      my %urlinfo = $self->getURL($path);
      my $url = $urlinfo{absurl};
      $attr->{$attribute} = "##P#R#E#F#I#X##TIME#$self->{timenum}##MODE#$mode##URL#$url##";
  }      
  $s .= $self->MakeTag($tag, $attr, $attrseq);
  $self->{callback}->($s);

} # end of sub handleAreaLinkScriptFrameIframeLayerIlayer ($$$$$) { 




sub handleApplet ($$$$$) { 
# Handling of tags APPLET
# may not be possible at all!!!

  my ($self, $tag, $attr, $attrseq, $origtext, %paths) = @_;
  my $s = '';
  my $path;
  my $mode = 'inline';
  my $codebase_attribute =  $paths{codebase} || "";
  my $archive_attribute = $paths{archive} || "";
  my $code_attribute = $paths{code} || "";

  ########### strategy for handling this tag:
  ### disable: codebase, archive
  ### set code=$codebase + code 
  ###
  ### The disabling is done by 

  ### modify each of the linkelements contained by the tag

  foreach my $attribute (keys %paths) {
      $path = $paths{$attribute};
      my %urlinfo = $self->getURL($path);
      my $url = $urlinfo{absurl};
      $attr->{$attribute} = "##P#R#E#F#I#X##TIME#$self->{timenum}##MODE#$mode##URL#$url##";
  }      
  $s .= $self->MakeTag($tag, $attr, $attrseq);
  $self->{callback}->($s);

} # end of sub handleApplet ()

sub handleForm($$$$$) { 
# Handling of Forms
#TODO: Decide whether to disable these forms
    
  my ($self, $tag, $attr, $attrseq, $origtext, %paths) = @_;
  my $s = '';
  my $path;
  my $mode = 'standalone'; # is this correct ?

  ### modify each of the linkelements contained by the tag

  foreach my $attribute (keys %paths) {
      $path = $paths{$attribute};
      my %urlinfo = $self->getURL($path);
      my $url = $urlinfo{absurl};
      $attr->{$attribute} = "##P#R#E#F#I#X##TIME#$self->{timenum}##MODE#$mode##URL#$url##";
  }      
  $s .= $self->MakeTag($tag, $attr, $attrseq);
  $self->{callback}->($s);

} # end of sub handleForm()

1;

=head1 NAME

HTMLDocParser.pm - parse HTML documents in the HTML-Document Handler

=head1 SYNOPSIS

use NWA::HTML::HTMLDokParser ();
    NWA::HTML::HTMLDokParser->new(
      'callback' => <reference to callback function>,
      'url' => <absolute url for Html-doc>,
      'callback'  => <Reference to callback function needed by 
		     the HTML::Parser>

=head1 DESCRIPTION

B<HTMLDokParser.pm> module is derived from B<HTML::Parser> and is an 
superclass for parsing HTML-documents.

=head1 METHODS

C<new>: A HTMLDocParser object is initialized with C<new()> with 
arguments 'callback' (a reference to callback funktion needed by 
the HTML::Parser) and 'url' (absolute url of the document):

    NWA::HTML::HTMLDocParser->new('callback' => <function>,
                                  'url' => <url>);

C<getURL>: Return a hash with information about an URL. 
C<extractURLs>: Return a hash with information about the URLs contained
in a certain tag.  

Then there is a lot of methods that override the methods in HTML::Parser: 
B<start>, B<end>, B<comment> and B<declaration>

Then we have some auxiliary functions: B<ElmEndStackPush>, B<ElmEndStackPop>, B<MakeTag>, B<extractCodebase>,
B<split_url>, B<process_tag>

Finally we have some functions to handle the different tags:


B<handleImgTableTrTdThA> handles the following tags: IMG, TABLE, TR, TD, TH and A  

B<handleApplet> handles the <Applet> tag: may not be possible to implement?

B<handleAreaLinkScriptFrameIframeLayerIlayer> handles the following tags: AREA, LINK, SCRIPT, FRAME, IFRAME, LAYER and ILAYER 

B<handleBase> handles the <BASE> tag

B<handleBgsound> handles the <BGSOUND> tag
 'blockquote' => \&handleBlockquote,

B<handleBody> handles the <BODY> tag.
 'del'     => \&handleDel,
 'embed'   => \&handleEmbed,
 'form'    => \&handleForm,
 'input'   => \&handleInput,
 'ins'     => \&handleIns,
 'isindex' => \&handleIsindex,
 'head'    => \&handleHead,
 'object'  => \&handleObject,
 'q'       => \&handleQ,
 'xmp'     => \&handleXmp,
 'param'   => \&handleParam,


=cut 
# End of $Id: HTMLDocParser.pm,v 1.2 2004/10/04 08:51:13 johnerik Exp $
