#!/usr/bin/perl 

##
## This file is part of The NWA Toolset.
##
## Copyright (C) 2001-2003 Royal Library in Stockholm,
##                         Royal Library in Copenhagen,
##                         Helsinki University Library in Finland,
##                         National Library of Norway,
##                         National and University Library of Iceland.
##
## The NWA Toolset if free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the free software foundation; either version 2 of the License, or
## (at your option) any later version.
##
## The NWA Toolset is distributed in the hope that it wil be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with The NWA Toolset; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

# $Id: install_nwatoolset.pl,v 1.10 2004/10/21 13:08:26 sverreb Exp $

# Require this version of Perl only if Exporter is being installed
#require 5.8.2;
use ExtUtils::MakeMaker;
#use ExtUtils::Packlist;
use Cwd;

$localhost = `hostname`;
chomp($localhost);
$message =  `ping -c 1 $localhost`;

print "Type in your host name or leave blank for default value, default is [$localhost]:";
chomp($input = <STDIN>);
if (length($input)>0){$localhost = $input};

$php_dir = "/var/www/html";
$nedlib_retriever_url = "http://$localhost/nwatoolset/retriever/retriever_nedlib.php";
$exporter_install_dir = "/usr/local/nwatoolset";
# Place test archive here
$archive_dir = "/tmp";
$access_conf_rootpath = "$php_dir";
$servlet_container_app_dir = "";

print <<'END';

NWA TOOLSET INSTALLATION

This installation script will install the NWA Toolset or
selected NWA Toolset components.

If you want to distribute the different components on different
machines choose the components to install on this machine and re-execute
the install script on the other machine(s) where you want 
the other component(s).

Note that the NWA Toolset requires a search engine as well. 
See the Installation Guide for information on how to install
the NWA adapted Lucene Search Engine or see System Configuration Guide
on how to interface your own search engine of choice.

Type in numbers of components you want to install:

1. Retriever 
2. Exporter
3. Access module 
4. Lucene search engine 

END

chop(my $mainmenu = <STDIN>);

if ($mainmenu =~ /1/) {
    &install_retriever;
}

if ($mainmenu =~ /2/){
    &check_prereqs_for_exporter;
    &install_exporter_conf;
    &install_exporter_pl;
    &install_exporter_utils;
    
}

if ($mainmenu =~ /3/){
    &install_access;
}

if ($mainmenu =~ /4/){
    &install_lucene_engine;
}

exit;

#---------------------------------------------------------
#---------------------------------------------------------

sub install_retriever {
print <<'END';

DOCUMENT RETRIEVER INSTALLATION

Type in numbers of components you want to install:

1. Nedlib Retriever 
2. ARC Retriever

END

chop(my $mainmenu = <STDIN>);

if ($mainmenu =~ /1/) {
    &install_nedlib_retriever;
}

if ($mainmenu =~ /2/) {
    &install_arc_retriever;
}
}

#---------------------------------------------------------
#---------------------------------------------------------

sub install_nedlib_retriever {

    print <<'END';

STARTING INSTALLATION OF NEDLIB RETRIEVER

For each question during installation, hit Enter to accept 
a default value presented inside [], or type in a new value.

END

print "\nThe Retriever should be installed into a directory";
print " where you can execute scripts.";
print "\nType in the name of a suitable directory.\n";
print "The Retriever will be installed in the directory 'nwatoolset/retriever/' relative to the given directory.\n";
print "Default is [$php_dir]:";
chomp($input = <STDIN>);
if (length($input)>0){$php_dir = $input};
$nedlib_retriever_dir = "$php_dir/nwatoolset/retriever";
$nedlib_retriever_dir = check_directory($nedlib_retriever_dir);
system "tar -xf retriever.tar -C $nedlib_retriever_dir";
print "Nedlib Retriever has been installed into $nedlib_retriever_dir.\n";
print "Please note that the retriever has to be configured manually. See documentation for details.\n";

print "\nFinished installation of Retriever\n\n";
}

#-------------------------------------------------------
#---- ARC RETRIEVER ------------------------------------

sub install_arc_retriever {

    print <<'END';

STARTING INSTALLATION OF ARC RETRIEVER

END



if (length(&resolve_servlet_container_app_dir())>0) {

    if (-e "ArcRetriever.war") {
	system("cp ArcRetriever.war $servlet_container_app_dir/");
        print "Copied the Nwa Arc Retriever Web application to $servlet_container_app_dir\n";
    } else {
        print "There's no ArcRetriever.war file available\n\n";
    }
} else {
    print "\nNo servlet container was given.\n";
    print "The 'ArcRetriever.war' file may be copied manually into the deployment directory\n";
    print "after a servlet container has been installed.\n";
}
}

#--------------------------------------------------
#--------------------------------------------------
sub check_prereqs_for_exporter {

###################################################
# ExtUtil::MakeMaker will write a Makefile, but
# that file is not used. Here it's just a side 
# effect of MakeMaker. We need MakeMaker to check 
# that some Perl modules needed by Exporter
# have been preinstalled.
###################################################

require 5.8.2;
print "\nCHECKING MODULES IN YOUR PERL INSTALLATION\n";
WriteMakefile (
   'NAME' => 'Nwatoolset',
   PREREQ_PM => {
       'LWP::Simple' => undef,
       'URI' => undef,
       'HTML::TokeParser' => undef,
       'XML::TokeParser' => undef,
       'XML::Parser' => undef,
       'XML::Simple' => undef,
       'Config::General' => undef,
       'Getopt::Std' => undef,
       'POSIX' => undef
   }
);

system "rm Makefile";
}
#---------------------------------------------------------
#---------------------------------------------------------

sub install_exporter_conf {

print<<'END';

STARTING INSTALLATION OF EXPORTER

For each question during installation, hit Enter to accept a default 
value presented inside [], or type in a new value.

END

print "\nType in installation directory for Exporter. 
This directory will by default contain subdirectories for 
source code, configuration and log files 
[default $exporter_install_dir]:";
chomp($input = <STDIN>);
if (length($input)>0) {$exporter_install_dir = $input};
$exporter_install_dir = check_directory($exporter_install_dir);

$exporter_bin_dir = $exporter_install_dir . "/bin";
check_directory($exporter_bin_dir);

$exporter_conf_dir = $exporter_install_dir . "/conf";
check_directory($exporter_conf_dir);

$exporter_log_dir = $exporter_install_dir . "/log";
check_directory($exporter_log_dir);

$exporter_log_file = $exporter_log_dir . "/Exporter.log";

if (-e "exporter.tar"){
     # Copy here because most files go to conf-directory.
     system "tar -xf exporter.tar -C $exporter_conf_dir";
     system "mv $exporter_conf_dir/exporter.pl.template $exporter_conf_dir/langid_wrap_textcat $exporter_conf_dir/pdftohtml_wrap $exporter_bin_dir";
}
else {
    print "There's no file exporter.tar present. 
I can't continue with Exporter installation. Returning ..\n";
    return;
}

# Open template file 'processor.conf.template' and read
# name-value pairs. Suggest these as defaults for the
# configuration file 'processor.conf'

open(TEMPLATE, "<$exporter_conf_dir/exporter.conf.template");
open(CONF, ">$exporter_conf_dir/exporter.conf");
while (<TEMPLATE>) {
    # Skip comment lines
    next if ($_ =~ /#/s);
    chomp;
    # Look for name-value pairs
    if ($_ =~ /(.+)\=(.+)/s){
	$name = $1;
	$value = $2;
        $name =~ tr/ //d;
        $value =~ tr/ //d;

#---- exporter.conf : general settings -------------------------
        if ($name eq "logfile"){
            print CONF "$name = $exporter_log_file\n\n";
        }
        elsif ($name eq "retriever_type"){
            print CONF "$name = $value\n\n";
         }        
        elsif ($name eq "retriever_url"){
#            unless ($mainmenu =~ /1/){
                print "\nExporter gets documents from Web archive through Document Retriever. 
Type in URL for the Retriever, usually it should be something like 
http://localhost/nwatoolset/docretriever_nedlib.php 
if you have installed Retriever into an nwatoolset subdirectory 
under php directory on your machine. 
Default for Retriever URL is [$nedlib_retriever_url]:";
                chomp($input = <STDIN>);
                if (length($input)>0) {$nedlib_retriever_url = $input};
 #           }
            print CONF "$name = $nedlib_retriever_url\n\n";
            }
        elsif ($name eq "nwa_dir"){
           print "\nType in the absolute name of the nwa directory into which 
Exporter will write nwa file(s)
default is [$value]:";
           chomp($input = <STDIN>);
           if (length($input)>0) {$value = $input};
           $value = check_directory($value);
           print CONF "$name = $value\n\n";
        }
        elsif ($name eq "collection_name"){
            print "\nType in the name of your collection
default is [$value]:";
            chomp($input = <STDIN>);
            if (length($input)>0){$value = $input};
            print CONF "$name = $value\n\n";
        }
        elsif ($name eq "lang_detection_on"){
            print CONF "$name = -1\n\n";
        }
        elsif ($name eq "document_conversion_on"){
            print CONF "$name = -1\n\n";
        }
        elsif ($name eq "document_conversion_handlers"){
            $value = $exporter_conf_dir . "/conversion_handlers_by_filetype";
            print CONF "$name = $value\n\n";
        } 
        elsif ($name eq "lang_detection_wrapper"){
            print CONF "$name = $exporter_install_dir/bin/langid_wrap_textcat\n\n";
        }
	elsif ($name eq "diskretriever_path") {
        }
        else{
            print CONF "$name = $value\n\n";
        }
    }
}

close(TEMPLATE);
close(CONF);
}

#-------------------------------------------------------------
#-------------------------------------------------------------

sub install_exporter_pl {

# Open template file 'exporter.pl.template' and
# change the value of $config_dir

open(TEMPLATE, "<$exporter_bin_dir/exporter.pl.template");
open(EXPORTER, ">$exporter_bin_dir/exporter.pl");
$config_dir = $exporter_conf_dir;
while (my $line = <TEMPLATE>) {
    if ($line =~ /CONFIG_DIR/){
	$line =~ s/CONFIG_DIR/$config_dir/;
    }
    print EXPORTER $line;
}
close(TEMPLATE);
close(EXPORTER);

print "Finished installation of Exporter files. They were placed into
$exporter_bin_dir and $exporter_conf_dir\n\n";

print "The Exporter's log file will be placed in the directory $exporter_log_dir\n\n";

}

sub install_exporter_utils {

## install utils for generating Archive-identifiers to feed the Exporter
## These utils are assumed to be in the utils.tar file

if (-e "utils.tar"){
     # Copy here because all files go to $exporter_bin_dir directory.
     system "tar -xf utils.tar -C $exporter_bin_dir";
     #system "mv $exporter_conf_dir/exporter.pl.template $exporter_conf_dir/gen_aids.pl $exporter_conf_dir/langid_wrap_textcat $exporter_conf_dir/pdftohtml_wrap $exporter_bin_dir";
}
else {
    print "Warning: There's no file utils.tar present. 
     The utilities will be missing from the Exporter installation. Returning ..\n";
    return;
}

}

#--------------------------------------------------------
#---- ACCESS MODULE -------------------------------------

sub install_access {

    print <<'END';

STARTING INSTALLATION OF ACCESS MODULE

For each question during installation, hit Enter to accept
a default value presented inside [], or type in a new value.

END

$conf_rootpath = $access_conf_rootpath;
print "\nType in rootpath for Access module. The Access module will be
installed into 'rootpath/nwatoolset'
default for rootpath is [$conf_rootpath]:";
chomp($input = <STDIN>);
if (length($input)>0){$conf_rootpath = $input};
$access_install_dir = $conf_rootpath . "/nwatoolset";
$access_install_dir = check_directory($access_install_dir);
if (-e "accessmodule.tar"){
    system "tar -xf accessmodule.tar -C $access_install_dir";
    system "mkdir $access_install_dir/manual";
    system "tar -xf manual.tar -C $access_install_dir/manual";
}
else {
    print "There's no file accessmodule.tar present.
I can't continue with installation of Access module. Returning ..\n\n";
    return;
}

# Ask for a searchengine

$engine = "Lucene";
#print "\nWhich searchengine will you use? You could choose 
#either FAST or Lucene 
#default is [$engine]:\n";
#chomp($input = <STDIN>);
#if (length($input)>0) {$engine = $input}; 


# Ask for a document retriever

#$retriever = "ArcRetriever";
#print "\nWhich document retriever will you use? You could choose 
#either ArcRetriver or NedlibRetriever 
#default is [$retriever]:\n";
#chomp($input = <STDIN>);
#if (length($input)>0) {$retriver = $input}; 


# Create config.inc from template
$document_retriever = $nedlib_retriever_url;
$conf_http_host = "http://$localhost/nwatoolset";
$conf_location_code = "test";
$conf_javascript = "off";

unless ($mainmenu =~ /2/){
    print "\nType in the URL to Document Retriever,
default is [$document_retriever]:";
    chomp($input = <STDIN>);
    if (length($input)>0) {$document_retriever = $input};
}

print "\nType in URL for this installation,
default is [$conf_http_host]:";
chomp($input = <STDIN>);
if (length($input)>0) {$conf_http_host = $input};

print "\nType in the name of your collection (same as used in Exporter installation),
default is [$conf_location_code]:";
chomp($input = <STDIN>);
if (length($input)>0){$conf_location_code = $input};

open(TEMPLATE, "<$access_install_dir/config.inc.template");
open(CONF, ">$access_install_dir/config.inc");

while (my $line = <TEMPLATE>) {
    if ($line =~ /\$conf\_rootpath\ *=\ *"(.+)"/){
        $line =~ s/$1/$access_install_dir/;
    }

    if ($engine =~ /lucene/i and $line =~ /\$conf\_searchenginepath\/(fast)/){
        $line =~ s/$1/lucene/;
    }

    if ($engine =~ /lucene/i and $line =~ /\/(fastSearch\.inc)/){
        $line =~ s/$1/luceneSearch.inc/;
    }

    if ($engine =~ /lucene/i and $line =~ /(\"fastSearch\")/){
        $line =~ s/$1/luceneSearch/;
    }

    if ($line =~ /\$document\_retriever\ *=\ *"(.+)"/){
        $line =~ s/$1/$document_retriever/;
    }

    if ($line =~ /\$conf\_http\_host\ *=\ *"(.+)"/){
        $line =~ s/$1/$conf_http_host/;
    }

    if ($line =~ /\$conf\_location\_code\ *=\ *"(.+)"/){
        $line =~ s/$1/$conf_location_code/;
    }

    if ($line =~ /\$conf\_javascript\ *=\ *"(.+)"/){
        $line =~ s/$1/$conf_javascript/;
    }

    if ($line =~ /\$conf\_locations\[\'(no)\'\]/){
        $line =~ s/$1/$conf_location_code/;
    }

    print CONF $line;
}
close(TEMPLATE);
close(CONF);

open (TEMPLATE, "<$access_install_dir/handlers/htmlhandler.pl.template");
open (CONF, ">$access_install_dir/handlers/htmlhandler.pl");
$perl_installation = `which perl`;
while( my $line = <TEMPLATE>){
    if ($line =~ /ABSOLUTE_PATH_TO_PERL/){
        $line =~ s/ABSOLUTE_PATH_TO_PERL/$perl_installation/;
    }
    if ($line =~ /(\/PATH_TO_NWA_MODULE)/){
        $line =~ s/$1/$access_install_dir\/handlers\/NWA/;
    }
    print CONF $line;
}
close(TEMPLATE);
close (CONF);

if ($engine =~ /lucene/i){
   # Create luceneconfig.inc from template
   $luceneconf_searchengineurl = "http://$localhost:8080/nwa/servlet/nwa"; 
   print "\nYou are using Lucene search engine. Type in URL for 
luceneconf_searchengineurl, or leave blank for default
default is [$luceneconf_searchengineurl]:";
   chomp($input = <STDIN>);
   if (length($input)>0) {$luceneconf_searchengineurl = $input};

   open(TEMPLATE, "<$access_install_dir/php_includes/seal/lucene/luceneconfig.inc.template");
   open(CONF, ">$access_install_dir/php_includes/seal/lucene/luceneconfig.inc");
   while (my $line = <TEMPLATE>) {
       if ($line =~ /\$luceneconf\_searchengineurl\ *=\ *"(.+)"/){
	   $line =~ s/$1/$luceneconf_searchengineurl/;
       }
       print CONF $line;
   }
   close(TEMPLATE);
   close(CONF);
}
    print "\nInstalled Access module into $access_install_dir\n\n";
}

#-------------------------------------------------------
#---- LUCENE SEARCH ENGINE -----------------------------

sub install_lucene_engine{

    print <<'END';

STARTING INSTALLATION OF LUCENE SEARCH ENGINE

END


if (length(&resolve_servlet_container_app_dir())>0) {

    if (-e "nwa.war") {
	system("cp nwa.war $servlet_container_app_dir/");
        print "Copied the Nwa Lucene Web application to $servlet_container_app_dir\n";
    } else {
        print "There's no nwa.war file available\n\n";
    }
} else {
    print "\nNo servlet container was given.\n";
    print "The 'nwa.war' file may be copied manually into the deployment directory\n";
    print "after a servlet container has been installed.\n";
}
}

#---------------------------------------------------------
#---------------------------------------------------------

sub check_directory{

# The input parameter is a directory name.
# Try to create this directory if it doesn't exist yet.

    my $dir_name = shift(@_);
    # Remove possible "/" from the end of directory name so
    # that all names are handled in a uniform way.
    if ($dir_name =~ /(.+)\/$/){$dir_name = $1}
    unless (-d $dir_name){
	print "Creating directory $dir_name\n\n";
	    system "mkdir -p $dir_name";
	    unless (-d $dir_name){
		print "Warning: Couldn't create $dir_name! Exiting now\n";
		exit;
	    }
    }
    return $dir_name;
}

#-----------------------------------------------------------
#-----------------------------------------------------------

sub resolve_servlet_container_app_dir{

    chomp($servlet_container_app_dir);
    if (length($servlet_container_app_dir)>0) {
	# Container dir already set, just return it.
	return $servlet_container_app_dir;
    };

    # If the environment variable CATALINA_HOME is set, try to
    # prefill $servlet_container_app_dir with the default webapp
    # path.

    if (length($ENV{'CATALINA_HOME'})>0) {
	$servlet_container_app_dir = "$ENV{'CATALINA_HOME'}/webapps";
    }

    print <<'END';

This module needs the existence of a servlet container (eg. Jakarta Tomcat).
Type in the full path to where 'war' files should be deployed for your
container. For Tomcat this will typically be $Tomcat_install_dir/webapps.

END

    print "Type in full path to where 'war' files should be deployed or leave blank for default value, default is [$servlet_container_app_dir]:";
    chomp($input = <STDIN>);
    if (length($input)>0){$servlet_container_app_dir = $input};

    return $servlet_container_app_dir;
}

#-----------------------------------------------------------
#-----------------------------------------------------------
