<?php
/*
 *  This file is part of The NWA Toolset.
 *
 *  Copyright (C) 2001-2002 Royal Library in Stockholm,
 *                          Royal Library in Copenhagen,
 *                          Helsinki University Library of Finland, 
 *                          National Library of Norway,
 *                          National and University Library of Iceland.
 *
 *  The NWA Toolset is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The NWA Toolset is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with The NWA Toolset; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
*  Class for building queries compliant with the 
*  NWA Search Engine Abstraction Layer (see indexSearch class)
*
*  Enables you to specify which indexes to query
*  and how they should be queried (i.e. for all or any words/phrases in query)
*  To query for phrases enclose the phrases in between " e.g. "the phrase"
*
* $Id: queryParser.inc,v 1.2 2004/07/04 21:22:02 sverreb Exp $
*/
class queryParser{
  
  var $parsedquery;
  
  /**
  * Constructor
  *
  * Use this this to input the "raw" query, the query type (all/any)
  * and the indexes (fields) to query.
  *
  * Usage:
  *
  * $myqueryparser = new queryParser('hamlet "to be or not to be" shakespeare', array ("nwa.text", "nwa.dctitle"), "all");
  * $myparsedquery = $myqueryparser->getParsedQuery();
  * 
  * Contents of $myparsedquery:
  *  (nwa.text:hamlet or nwa.dctitle:hamlet) and
  *  (nwa.text:"to be or not to be" or nwa.dctitle:"to be or not to be") and
  *  (nwa.text:shakespeare or nwa.dctitle:shakespeare)
  *
  * @param  $string  The "raw" query
  * @param  $array   The indexes to query
  * @param  $string  The query type, legal values: any, all (default)
  */
  function queryParser($query, $fields_to_query, $querytype) {

    switch ($querytype) {
      case "any":
        $operator = "or";
        break;
      default: // "all"
        $operator = "and";
    }
    
    // Replace the spaces within the phrases with @ to avoid the 
    // explode function to tear them apart
    $words_in_query = explode(' ', stripslashes(preg_replace("/(\")([^\"]*)(\")/e", "'\\1'.str_replace(' ', '@', '\\2').'\\3'", $query)));

    $num_words = 0;
    foreach ($words_in_query as $word) {
      if ($word != "") { // Do nothing with empty elements caused by multiple spaces between words/phrases
        $num_words++;
        $word = str_replace ("@", " ", $word);
        $querytemp = "(";
        foreach ($fields_to_query as $field) {
          if ($querytemp == "(") {
            $querytemp .= $field . ":" . $word;
          }
          else {
            $querytemp .= " or " . $field . ":" . $word;
          }
        }
        $querytemp .= ")";
        if ($querystring == "") {
          $querystring .= $querytemp;
        }
        else {
          $querystring .= " ". $operator . " " . $querytemp;
        }
      }
    }
    if ($num_words == 1) {
      // Omit parantheses if only one word/phrase in query
      $querystring = strtr($querystring, "()", "  ");
    }
    $this->parsedquery = $querystring;
  }

  /**
  * Retrieve the parsed query
  *
  * @return string The parsed query
  */
  function getParsedQuery() {
    return $this->parsedquery;
  }
}
?>