<?php
/*
 *  This file is part of The NWA Toolset.
 *
 *  Copyright (C) 2001-2002 Royal Library in Stockholm,
 *                          Royal Library in Copenhagen,
 *                          Helsinki University Library of Finland, 
 *                          National Library of Norway,
 *                          National and University Library of Iceland.
 *
 *  The NWA Toolset is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The NWA Toolset is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with The NWA Toolset; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
* Class for generating an array of unique search results
* i.e. documents with the same url is not listed more than once
*
* $Id: resultlist.inc,v 1.3 2004/10/04 09:04:48 johnerik Exp $
*/

class resultList {

  var $indexdriver;
  var $resultset = array();
  var $errormessage;
  var $query;
  var $vquery;
  var $start;
  var $end;
  var $hitsperquery;
  var $numhitstotal;
  var $firstkey;
  var $lastkey;
  var $link_to_next;
  
  /**
  * Constructor
  *
  * @param   object  Instance of search engine class
  * @param   string  querystring
  * @param   integer result set start
  * @param   integer size of result set
  * @param   string  sort order
  * @param   string  fields to return in result set, comma separated
  * @return  string  Result set
  */  
  function resultList($indexdriver, $query, $start, $sizeofresultset, $sortorder, $fields) {
    
    $this->link_to_next = true;
    $this->indexdriver = $indexdriver;
    $this->query = $query;
    $this->start = $start;
    $this->end = $this->start + $sizeofresultset - 1;
    $this->hitsperquery = 10;
    $this->fields = $fields;
    $this->sortorder = $sortorder;
    
    $this->indexdriver->setQuery($this->query);
    $this->indexdriver->setSortorder($this->sortorder);
    $this->indexdriver->setSizeOfResultSet($this->hitsperquery);
    $this->indexdriver->setFieldsInResult("url");
    if ($this->indexdriver->doQuery()) {
      $numhits = $this->indexdriver->getnumhits();
      $this->numhitstotal = $this->indexdriver->getNumHitsTotal();
      $this->resultset = $this->indexdriver->getResultSet();
    }
    else {
      $this->numhitstotal = -1;
    }
  }
  
// Private methods

  /**
 * Get a result set containing only unique url's
 *
 * @return array result set 
 */
  function getUniqueHits() {

    $tmp_resultset = $this->indexdriver->getResultSet();
    $hit = 1;
    $queryno = 1;

    for ($r = 1; $hit <= $this->end and $r <= $this->numhitstotal; $r++) {
      if ($r == $this->numhitstotal) {
        $this->link_to_next = false;
      }
      if ($r == $queryno * $this->hitsperquery + 1) {
        $this->indexdriver->setOffset($queryno * $this->hitsperquery);
        $this->indexdriver->doQuery();
        $tmp_resultset = $this->indexdriver->getResultSet();
        $queryno++;
      }    
      $duplicate = 0;
      if ($hit > 1) {  
        for ($rprev = 1; $rprev < $r; $rprev++) {
          if ($tmp_resultset[$r]["url"] == $tmp_resultset[$rprev]["url"]) {
            $duplicate = 1;
            break; // No need to find more than one duplicate
          }
        }
      }
      if ($duplicate == 0) {
        if ($hit >= $this->start) {
          $resultset[$hit] = $tmp_resultset[$r];
          $resultset[$hit]['no'] = $r;
          if (empty($this->firstkey)) {
            $this->firstkey = $r;
          }
          $this->lastkey = $r;
        }
        $hit++;
      }
    }
    return $resultset;
  }


// Public methods

  /** 
  * Get total number of versions
  *
  * @return number of hits
  */  
  function getNumVersions() {
    return $this->numhitstotal;
  }

  /**
 * Get result set
 *
 * @return array result set 
 */
  function getResultSet() {
  
    $resultset1 = $this->getUniqueHits();
    
    unset ($this->indexdriver->resultset);
    $this->indexdriver->setSizeOfResultSet($this->lastkey - $this->firstkey + 1);
    $this->indexdriver->setOffset($this->firstkey - 1);
    $this->indexdriver->setFieldsInResult($this->fields);
    $this->indexdriver->doQuery();
    $resultset2 = $this->indexdriver->getResultSet();

    foreach ($resultset1 as $key => $val) {
      

      $resultset[$key] = $resultset2[$resultset1[$key]["no"]];
      unset ($this->indexdriver->resultset);

      $vquery = "(" . $this->query . ") and nwa.url:" . $resultset1[$key]["url"];
      $this->indexdriver->setQuery($vquery);
      $this->indexdriver->setSortorder("descending");
      $this->indexdriver->setSizeOfResultSet(1000);
      $this->indexdriver->setOffset(0);
      $this->indexdriver->setFieldsInResult(archival_time);
      $this->indexdriver->doQuery();
      $versions = $this->indexdriver->getResultSet();
     
      $resultset[$key]["numversions"] = $this->indexdriver->getNumHitsTotal();
      $resultset[$key]["versions"] = $versions;
      
      $this->indexdriver->setQuery("nwa.url:" . $resultset1[$key]["url"]);
      $this->indexdriver->doQuery();
      $resultset[$key]["totalversions"] = $this->indexdriver->getNumHitsTotal();
      $hits_in_set = $key;
      
      $last_hit_no_in_set = $resultset1[$key]["no"] + $resultset[$key]["numversions"] - 1;
      
    }

    $resultset['hits_in_set'] = $hits_in_set - $this->start + 1;
    $resultset['last_hit_no_in_set'] = $last_hit_no_in_set;

    return $resultset;
  }
 
  /**
 * Get error message
 *
 * @return string Error message
 */
  function getErrorMessage() {
    return $this->indexdriver->getErrorMessage();
  }
}
?>
