<?php
/*
 *  This file is part of The NWA Toolset.
 *
 *  Copyright (C) 2001-2002 Royal Library in Stockholm,
 *                          Royal Library in Copenhagen,
 *                          Helsinki University Library of Finland, 
 *                          National Library of Norway,
 *                          National and University Library of Iceland.
 *
 *  The NWA Toolset is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The NWA Toolset is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with The NWA Toolset; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

include("$conf_searchenginepath/indexSearch.inc");
include("$conf_searchenginepath/indexUtils.inc");

/**
* Class for querying a FAST Search Engine
*
* For further information about constructing 
* queries and setting parameters for the fastSearch
* class see the FAST Site Search Development 
* Guide 3.0.0
*
* $Id: fastSearch.inc,v 1.1.1.1 2004/02/04 14:29:09 sverreb Exp $
*/
class fastSearch extends indexSearch {
  // Fast specific variables
  var $dispatchnodeurl;
  var $querytype;
  var $rankprofileid;
  var $host;
  var $param;
  var $keyoflastresultfieldtofetch;
  var $summaryId;
  var $legalsortordervalues = array();
  var $legalresultfields = array();
  var $hosturl;
  var $tvm_query;
  var $tvm_query_esc; 
  var $tvm_query_enc; 
  var $tvm_query_type;
  var $debugstring;
  var $pattern;
  var $replacement;
  var $fastadaptedquery;
  var $http_api_resultformat;
  
  /**
  * Constructor. 
  * Set some initial values 
  */
  function fastSearch() {
    include ("fastconfig.inc");    
    $this->dispatchnodeurl = $fastconf_dispatchnodeurl;
    $this->http_api_resultformat = $fastconf_http_api_resultformat;
    $this->query = "";
    $this->numhits = 0;
    $this->hitsperset = 10;
    $this->offset = 0;
    $this->param = "";
    $this->timespent = 0;
    $this->setQueryType('adv');

  }

  
  
  // FAST SPECIFIC FUNCTIONS

  /**
  * Get legal result fields
  *
  * Use this to retrieve the legal result fields 
  * defined in the parameter file of the constructor
  *
  * @return array Legal result fields
  */
  function getLegalResultFields() {
    return $this->legalresultfields;
  }

  /**
  * Get legal sort order values
  *
  * Use this to retrieve the legal sort order values
  * defined in the parameter file of the constructor
  *
  * @return array Legal result fields
  */
  function getLegalSortOrderValues() {
    return $this->legalsortordervalues;
  }

  /** 
  * Set query type for search
  *  
  * Set the query type to one of the following
  * all : all terms must be found
  * any : any of the query terms may give a hit
  * phrase : the exact phrase must be found
  * adv : the query is an expression in the Fast advanced query language
  *
  * @param  integer Query type
  */
  function setQueryType($querytype) {
      $this->querytype = $querytype;
  }

  /**
  * Check if ready to execute query (query string non-empty)
  *
  * @return boolean true if query string non-empty, false otherwise
  */
  function isReady() {
    if ($this->query != "") {
      return true;
    }
    else {
      return false;
    }
  }

  /**
  * Set extra parameters unknown to the object
  *
  * @param  $string  Extra parameter
  */  
  function setExtraParams($string) {
    $this->param = $string;
  } 
    

  function adaptQueryToFastIndex($querystring) {

    /* 
    || FAST specific
    || ------------
    || Used for adapting NWA Search Engine Abstraction Layer queries to the FAST 
    || index structure. The parameters  are used in the setQuery metod. 
    || For non-FAST use uncomment the following lines
    || and the replacements done in the setQuery method.
    ||
    */    

    $querypattern1 = "/(nwa.url:|nwa.dcformat:|nwa.framelinks:|nwa.multimedialinks:|nwa.links:|nwa.archiveidentifier:)([^\ )]*)/e"; // FAST specific
    $queryreplacement1 = "'\\1'.index_encode('\\2')"; // FAST specific
    
    //$querypattern2 = "/(nwa.dcdate:)([^\ ]*)/e"; // FAST specific
    //$queryreplacement2 = "'\\1'.convert_time_to_int('\\2')"; // FAST specific    
    $querypattern2 = "/(nwa.dcdate:(\[){0,1})([0-9]*)((;){0,1})([0-9]*)((\]){0,1})/e";
    $queryreplacement2 = "'\\1'.convert_time_to_int('\\3').'\\4'.convert_time_to_int('\\6').'\\7'";

    
    $querypattern3 = array ("/nwa.dcdate/","/nwa.archiveidentifier/","/nwa.url/","/nwa.collection/","/nwa.encoding/","/nwa.framelinks/","/nwa.multimedialinks/","/nwa.links/","/nwa.linknames/","/nwa.text/","/nwa.dctitle/","/nwa.dcsubject/","/nwa.dcdescription/","/nwa.dccreator/","/nwa.dcpublisher/","/nwa.dccontributor/","/nwa.dctype/","/nwa.dcformat/","/nwa.dcidentifier/","/nwa.dcsource/","/nwa.dclanguage/","/nwa.dcrelation/","/nwa.dccoverage/","/nwa.dcrights/", "/INDX2A/"); // FAST specific
    $queryreplacement3 = array ("nwanumeric.dcdate","nwa1.archiveidentifier","nwa1.url","nwa1.collection","nwa1.encoding","nwa1.framelinks","nwa1.multimedialinks","nwa1.links","nwa1.linknames","nwa2.text","nwa2.dctitle","nwa2.dcsubject","nwa2.dcdescription","nwa2.dccreator","nwa2.dcpublisher","nwa2.dccontributor","nwa3.dctype","nwa3.dcformat","nwa3.dcidentifier","nwa3.dcsource","nwa3.dclanguage","nwa3.dcrelation","nwa3.dccoverage","nwa3.dcrights", "*"); // FAST specific

    $query = preg_replace("/%20/", "INDX20", $querystring);
    $query = preg_replace("/&amp;/", "&", $query);
    $query = preg_replace($querypattern1, $queryreplacement1, $query); // FAST specific
    $query = preg_replace($querypattern2, $queryreplacement2, $query); // FAST specific
    $query = preg_replace ($querypattern3, $queryreplacement3, $query); // FAST specific
    return $query;
   
  }
  
  // FUNCTIONS OVERRIDING PARENT CLASS FUNCTIONS
  
  /** 
  * Sets the sortorder 
  * legal values for sortorder is:
  * "relevance", "ascending" and "descending"
  */
  function setSortOrder($sortorder) {
    $this->sortorder=$sortorder;
    switch ($this->sortorder) {
      case "relevance":
        $this->rankprofileId=0;
        break;
      case "ascending":
        $this->rankprofileId=1;
        break;
      case "descending":
        $this->rankprofileId=2;
        break;
    }
  } 


  /**
  * Set result set fields
  *
  * Use this to set which summary fields to include 
  * in the result set returned by the getResultSet 
  * and getXmlResultSet methods.
  *
  * @param  string  Fields in result set
  * @return boolean False if fields not defined as legal result fields .
  */
  function setFieldsInResult($fields) {
    $this->resultfields = preg_split ("/[\s,]+/", trim($fields));
  }

  function doQuery() {
    if ($this->isReady()) {
      if ($this->http_api_resultformat == "php") { // For backward compatibility
        $retval = $this->doQueryForPhpResult();
      }
      else {
        $retval = $this->doQueryForTextResult();
      }
    }       
    else {
      $retval = false;
      $this->errormsg = "Empty query";
    }
    return $retval;
  }  
  
  /**
  * Execute the query
  *
  * @return boolean False if empty query
  */ 

  function doQueryForTextResult() {
      /*
      * The include below will return:
      *
      * TODO : Error handling (this implementation always return true!)
      */
      
      
      $lines = file ($this->dispatchnodeurl . "?query=" . urlencode($this->fastadaptedquery) . "&hits=$this->hitsperset&offset=$this->offset&type=$this->querytype&ranking=$this->rankprofileId");
      $fieldnames = array( 1 => "archiveidentifier", 2 => "url", 3 => "collection", 4 => "encoding", 5 => "framelinks", 6 => "multimedialinks", 7 => "links", 8 => "linknames", 9 => "teaser", 10 => "dcdate", 11 => "dctitle", 12 => "dccreator", 13 => "dcsubject", 14 => "dcdescription", 15 => "dcpublisher", 16 => "dccontributor", 17 => "dctype", 18 => "dcformat", 19 => "dcidentifier", 20 => "dcsource", 21 => "dclanguage", 22 => "dcrelation", 23 => "dccoverage", 24 => "dcrights");
      $tmp_array = explode("#*#*#", $lines[0]);
      $this->numhits = $tmp_array[0];
      $this->numhitstotal = $tmp_array[1];
      $this->timespent = $tmp_array[2];
    
      if ($this->numhits > 0) {
        $nn = 0;
        for ($hitno = $this->offset + 1; $hitno >= ($this->offset + 1) and $hitno <= ($this->offset + $this->hitsperset) and $hitno <= $this->numhitstotal; $hitno++) {
          $nn++;
          $fields[$hitno] = explode("#*#*#", $lines[$nn]);
          for ($i = 1; $i <= 23; $i++) {
            $tmp_resultset[$hitno][$fieldnames[$i]] = $fields[$hitno][$i];
          }
        }
        
        for ($i = $this->offset+1; $i <= $this->numhits+$this->offset; $i++) {
          reset($this->resultfields);
          while (list($key,$val)=each($this->resultfields)) {
            if ($val == "dcdate" and strlen($tmp_resultset[$i]["dcdate"]) != 14) { 
              // Convert dcdate from integer to human readable format
              $this->resultset[$i][$val] = gmdate('YmdHis', $tmp_resultset[$i][$val]); 
            }
            else { 
              $this->resultset[$i][$val] = $tmp_resultset[$i][$val];
            }
          }
        }
        
      } 
    return true;
  }

  function doQueryForPhpResult() {
      /*
      * Included for backward compatibility
      *
      * The include below will return:
      * $tvm_resultset[hitno][field]
      * $tvm_numhits - Number of hits in current set
      * $tvm_numhitstotal - Number of hits total 
      * $tvm_timespent - Time passed since the query was submitted
      * $tvm_errormsg - Error message returned by the search engine if an error occured
      * $tvm_query - The actual query string submitted
      * $tvm_query_esc - The query string, escaped for use in an URL according to the HTTP protocol
      * $tvm_query_enc - The query string, escaped for use in HTML text (e.g. "and" returns &quot;and&quot;)
      * $test->tvm_query_type - String decribing the type of search (i.e. "all", "any", "phrase" or "adv")
      */
      
      include($this->dispatchnodeurl . "?query=" . urlencode($this->fastadaptedquery) . "&hits=$this->hitsperset&offset=$this->offset&type=$this->querytype&ranking=$this->rankprofileId");
      if (!isset($tvm_numhits)) {
        $retval = false;
        $this->errormsg="Connection failed";
      }
      elseif (isset($tvm_errormsg)) {
        $retval = false;
        $this->errormsg=$tvm_errormsg;
      }
      else {
        $retval = true;  
        $this->numhits = $tvm_numhits;
        $this->numhitstotal = $tvm_numhitstotal;
        $this->timespent = $tvm_timespent;
        $this->tvm_query = $tvm_query;
        $this->tvm_query_esc  = $tvm_query_esc; 
        $this->tvm_query_enc  = $tvm_query_enc; 
        $this->tvm_query_type = $tvm_query_type;        
        for ($i = $this->offset+1; $i <= $this->numhits+$this->offset; $i++) {
          reset($this->resultfields);
          while (list($key,$val)=each($this->resultfields)) {
            if ($val == "dcdate" and strlen($tvm_resultset[$i]["dcdate"]) != 14) { 
              // Convert dcdate from integer to human readable format
              $this->resultset[$i][$val] = gmdate('YmdHis', $tvm_resultset[$i][$val]); 
            }
            /*
            elseif ($val == "url") {
              $this->resultset[$i][$val] = trim($tvm_resultset[$i][$val]);
            }*/
            else { 
              $this->resultset[$i][$val] = $tvm_resultset[$i][$val];
            }
          }
        }
      }
    return $retval;
  }

  /**
  * Set query string for search
  * 
  * @param  string  Query string
  */
  function setQuery($querystring) {
    $this->query = $querystring;
    $this->fastadaptedquery = $this->adaptQueryToFastIndex($this->query);
  }
}
?>