<?php
/*
 *  This file is part of The NWA Toolset.
 *
 *  Copyright (C) 2001-2002 Royal Library in Stockholm,
 *                          Royal Library in Copenhagen,
 *                          Helsinki University Library of Finland, 
 *                          National Library of Norway,
 *                          National and University Library of Iceland.
 *
 *  The NWA Toolset is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The NWA Toolset is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with The NWA Toolset; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

include("$conf_searchenginepath/indexSearch.inc");
include("$conf_searchenginepath/indexUtils.inc");

/**
* Class for querying a LUCENE Search Engine
*
* For further information about constructing 
* queries and setting parameters for the luceneSearch
* class see the Lucene Documentation
*
* $Id: luceneSearch.inc,v 1.5 2004/10/22 11:22:47 johnerik Exp $
*/
class luceneSearch extends indexSearch {
  // Lucene specific variables
  var $searchengineurl;
  var $legalsortordervalues = array();
  var $legalresultfields = array();
  var $pattern;
  var $replacement;
  var $ranking;
  
  /**
  * Constructor. 
  * Set some initial values 
  */
  function luceneSearch() {
    include ("luceneconfig.inc");    
    $this->searchengineurl = $luceneconf_searchengineurl;
    $this->resultfield_separator = $luceneconf_separator;
    $this->query = "";
    $this->numhits = 0;
    $this->hitsperset = 10;
    $this->offset = 0;
    $this->timespent = 0;
    $this->ranking = 0;

  }

  
  
  // LUCENE SPECIFIC FUNCTIONS

  /**
  * Check if ready to execute query (query string non-empty)
  *
  * @return boolean true if query string non-empty, false otherwise
  */
  function isReady() {
    if ($this->query != "") {
      return true;
    }
    else {
      return false;
    }
  }

  // FUNCTIONS OVERRIDING PARENT CLASS FUNCTIONS
  
  /** 
  * Sets the sortorder 
  * legal values for sortorder is:
  * "relevance", "ascending" and "descending"
  */
  function setSortOrder($sortorder) {
    $this->sortorder=$sortorder;
    switch ($this->sortorder) {
      case "relevance":
        $this->ranking=0;
        break;
      case "ascending":
        $this->ranking=1;
        break;
      case "descending":
        $this->ranking=2;
        break;
    }
  } 


  /**
  * Set result set fields
  *
  * Use this to set which summary fields to include 
  * in the result set returned by the getResultSet 
  * and getXmlResultSet methods.
  *
  * @param  string  Fields in result set
  * @return boolean False if fields not defined as legal result fields .
  */
  function setFieldsInResult($fields) {
    $this->resultfields = preg_split ("/[\s,]+/", trim($fields));
  }

  /**
  * Execute the query
  *
  * @return boolean False if empty query
  */ 

  function doQuery() {
    $this->query = trim($this->query);
    $this->query = str_replace(" and ", " AND ", $this->query);
    $this->query = str_replace(" or ", " OR ", $this->query);
    $this->query = str_replace(" not ", " NOT ", $this->query);

    if ($this->isReady()) {
      $resultfieldspart = "&fieldsinresult=" . urlencode(implode(",", $this->resultfields));
      $query = $this->searchengineurl . "?query=" . urlencode($this->query) . $resultfieldspart . "&hits=$this->hitsperset&offset=$this->offset&ranking=$this->ranking&separator=$this->resultfield_separator";

      $file = file_get_contents ($query);

      $all_fields = explode($this->resultfield_separator, $file);

      $this->numhits = $all_fields[0];
      $this->numhitstotal = $all_fields[1];
      $this->timespent = $all_fields[2];

      if ($this->numhits > 0) {
          for ($hitnum = 0; $hitnum < $this->numhits; $hitnum++) {
              $resultfields_count = count($this->resultfields);
              for ($fieldnum = 0; $fieldnum < $resultfields_count; $fieldnum++) {
                  $this->resultset[$hitnum + $this->offset + 1][$this->resultfields[$fieldnum]] =
                      $all_fields[4 + ($hitnum * ($resultfields_count + 1)) + $fieldnum];
              }
          }
      }

    return true;
    }
  }

  /**
  * Set query string for search
  * 
  * @param  string  Query string
  */
  function setQuery($querystring) {
    $this->query = $querystring;
  }
}
?>
