<?php
/*
 *  This file is part of The NWA Toolset.
 *
 *  Copyright (C) 2001-2002 Royal Library in Stockholm,
 *                          Royal Library in Copenhagen,
 *                          Helsinki University Library of Finland, 
 *                          National Library of Norway,
 *                          National and University Library of Iceland.
 *
 *  The NWA Toolset is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The NWA Toolset is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with The NWA Toolset; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/**
 * $Id: retriever_nedlib.php,v 1.5 2004/10/05 12:33:04 sverreb Exp $
 *
 * This script implements a Retriever for a Nedlib style archive.
 * Please note that security issues is not addressed in the implementation.
 * Normally the script will execute behind a firewall, giving the possibility 
 * to grant access only to the NWA Toolset Access Module and/or Exporter.
 * If you must make the script available for execution on a public website
 * please consider other access control mechanisms to prevent unauthorized 
 * access to your files.
 *
 * TODO : 
 *
 *
 */

$reqtype = $_GET['reqtype'];
$aid = $_GET['aid'];

//error_log("[retriever_nedlib.php]: Current AID: $aid");


// $conf_archive_directory sets the directory  where you archive files recide,
// (e.g. /archive/) to hinder retrieval of files in other areas of your disc

// IF your identifiers are relative paths, the
// paths are assumed to relative to the path set in 
// the $conf_archive_directory variable defined in the below inc-file

include ("conf_retriever_nedlib.inc");

// if relative AID, make it absolute before attempting to retrieve it.

if (preg_match("/^\//", $aid)){
  // this aid is absolute (i.e. starts with /)
  // No changes necessary
} else {
  // this aid is relative (i.e. does not start with /)
  // make it absolute by prefixing it with  $conf_archive_directory

  $aid = "$conf_archive_directory/$aid";
  error_log("[retriever_nedlib.php]: Aid made absolute by prefixing it with $conf_archive_directory:  $aid");
}


if (!isset($reqtype)) {
  $retrievermessage = buildErrorMessage(1);
}
else {
  $retrievermessage['head']['reqtype'] = $reqtype;

  if (!isset($aid) && $reqtype!="getarchiveinfo") {
    $retrievermessage = buildErrorMessage(2);
  }
  elseif ($reqtype!="getarchiveinfo" && !aidEval($aid)) {
    $retrievermessage = buildErrorMessage(4);
  }
  else {
    switch($reqtype) {
     
      case "getfile":
      if(!@readfile ($aid)) {
	// return XML-error message
	$retrievermessage = buildErrorMessage(5); // unable to read file identified by aid. 
	xmlOut($retrieverMessage);
	error_log("[retriever_nedlib.php]: Unable to read file '$aid'");
        //header("HTTP/1.0 404 Not Found");
      }
      exit;

      case "getmeta":
      $retrievermessage = getMeta($aid);
      break;

      case "getstatus":
      $retrievermessage['head']['aid'] = $aid;
      $retrievermessage['body']['status'] = getStatus($aid);
      break;
     
      case "getarchiveinfo":
      $retrievermessage['body']['info'] = 'This is the Web Archive of the National Library of Norway';
      break;
     
      default:
      $retrievermessage = buildErrorMessage(3);
    }
  }
}
//printDebug($retrievermessage);
xmlOut($retrievermessage);
exit; 

///// end of script get_document.php //////

///// functions used in script ////////

function printDebug($arr) {
  echo "<pre>";
  print_r($arr);
  echo "</pre>";
}

/** 
* Build error message 
*
* @param  integer  Error code
* @return array    Array containing error code and corresponding error message
*/
function buildErrorMessage($errcode) {
  $errcodearray = array( 
    1 => 'No request type given, legal values are: getmeta, getfile, getstatus and getarchiveinfo',
    2 => 'Missing archive identifier (aid)',
    3 => 'Illegal request type, legal values are: getmeta, getfile, getstatus and getarchiveinfo',
    4 => 'Illegal archive identifier (aid)',
    5 => 'Unable to read file identified by aid'
  );
  $error['head']['errorcode'] = $errcode;
  $error['head']['errormessage'] = $errcodearray[$errcode];
  return $error;
}

/** 
* Produce the status of a given file in the archive 
*
* @param  string    aid, The files Archive identifier
* @return string    Status, legal values are "online", "offline", "non-existent"
*/
function getStatus($aid) {
  // Super-simple implementation
  if (file_exists($aid . ".meta")) {
    if (file_exists($aid)) {
      $status = "online";
    }
    else {
      $status = "offline";
    }
  }
  else {
    $status = "non-existent";
  }
  return $status;
}

/** 
* Produce metadata for a given file in the archive  
*
* @param  integer  aid, The files Archive identifier
* @return array    Array containing the metadata
*/


function getMeta($aid) {
  $meta['head']['reqtype'] = 'getmeta';
  $meta['head']['aid'] = $aid;
  $meta['body']['metadata']['status'] = getStatus($aid);
  if ($meta['body']['metadata']['status'] = "online") {
    $meta['body']['metadata']['content_length'] = filesize($aid);
    $meta['body']['metadata']['content_checksum'] = md5_file($aid);
  }
  $fp = @fopen ($aid . ".meta", "r");
  if($fp) {
    while(!feof($fp)) {
      $line = fgets($fp, 4096);
      if(preg_match("/^Arc-Url/", $line)) {
        $url = preg_replace("/^Arc-Url: (.*)/", "$1", $line);
        $url = trim($url); // remove white-space before and after the URL
      } 
      else if(!preg_match("/^(Arc-|Meta-Data)/", $line)) {
        $header .= $line ;
      }
      
      if (preg_match("/(content-type *: *)([^ ;]*)(.*charset *= *)?([^ ;]*)/i", $line, $matches)) {
        $meta['body']['metadata']['contenttype']['type'] = $matches[2];
        $meta['body']['metadata']['contenttype']['charset'] = $matches[4];
      }

      if (preg_match("/(Last-Modified *:)(.*)/i", $line, $matches)) {
        $meta['body']['metadata']['last_modified_time'] = gmdate("YmdHis", strtotime($matches[2]));
      }

    }
    fclose($fp);
    
    // Cannot trust dates from http header, use timestamp of file instead (time of harvest)
    $meta['body']['metadata']['archival_time'] = gmdate('YmdHis', filemtime($aid . ".meta")); 
    
    $meta['body']['metadata']['url'] = cdatawrap(urldecode($url));
    $meta['body']['metadata']['http-header'] = cdatawrap($header);
  } 
  else {
    $meta['head']['errorcode'] = 3;
    $meta['head']['errormessage'] = "No metadata available for \"$aid\"";
  }
  return $meta;
}

/** 
* Convert a retriever message array to XML and print it out   
*
* @param  array   Retriever message array
*/
function xmlOut($arr) {
  header("Content-Type: text/xml");
  echo "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<retrievermessage>\n";  
  printArrayAsXml($arr);
  echo "</retrievermessage>";
}

/** 
* Convert a retriever message array to XML (used by xmlOut)   
*
* @param  array   Retriever message array
*/  
function printArrayAsXml($arr) {
  foreach ($arr as $key => $value) {
    if (is_array($value)) {
      echo "<$key>\n";
      printArrayAsXml($value);
      echo "</$key>\n";
    }
    else {
      echo "<$key>" . utf8_encode($value) . "</$key>\n";
   }
  }
}

/** 
* Wrap a string inside CDATA tags   
*
* @param  string   
* @return string
*/
function cdatawrap($text) {
  return "<![CDATA[$text]]>";
}

/** 
* Evaluate a given archive identifier
*
* @param  string    aid, archive identifier
* @return boolean   True if the aid is considered valid, false otherwise
*/
function aidEval($aid) {
  include ("conf_retriever_nedlib.inc"); //Also included above
  $retval = true;
  $posn = strpos($aid, $conf_archive_directory);
  // Should maybe be possible to specify several paths in the conf file 
  if ($posn === false) {
    $retval = false;
    error_log("[retriever_nedlib.php] AID $aid does not belong to the legal archive directory ($conf_archive_directory) set in the conf_retriever_nedlib.inc");
  }
  elseif (strlen(substr($aid, (strrpos($aid, "/") + 1))) != 32) {
    //Assumes that the filenames are kept unaltered since harvesting (Nedlib)  
    $retval = false;
    error_log("[retriever_nedlib.php]: AID $aid does not describe a document in the legal archive directory set in the conf_retriever_nedlib.inc");
  }
  return $retval;
}
?>
